/* sqtile.h -- Square-Figure tiling functions.
**
** Copyright (C) 2003 Eric Laroche.  All rights reserved.
**
** @author Eric Laroche <laroche@lrdev.com>
** @version @(#)$Id: sqtile.h,v 1.4 2003/04/10 09:07:27 laroche Exp $
**
** This program is free software;
** you can redistribute it and/or modify it.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*/


#ifndef SQTILE_H
#define SQTILE_H


#include "sqfig.h"


/** Board index from X/Y.
*/
#define BOARD(x, y, xDimension) ((x) + (y) * (xDimension))

/** X from board index.
*/
#define XBOARD(board, xDimension) ((board) % (xDimension))

/** Y from board index.
*/
#define YBOARD(board, xDimension) ((board) / (xDimension))


/** Get first tiling for a given X dimension.  The tiling buffer must be
* able to hold DISTINCT_FIGURE_COUNT(figureSet) ints, which will carry
* on return the figure indices.  The board buffer must be able to hold
* figureSize*DISTINCT_FIGURE_COUNT(figureSet) ints and will carry on
* return the board's tiling layout as indices into the tiling buffer.
* The return value indicates if a solution has been found (1) or not (0).
*/
int getFirstTiling(
	figureSet_t const* figureSet,
	int xDimension,
	int* tiling,
	int* board,
	int continuous,
	int skipSymmetrical);

/** Get next tiling for a given X dimension.  The tiling buffer must be
* able to hold DISTINCT_FIGURE_COUNT(figureSet) ints, which must have
* been filled by a previous succeeding getFirstTiling() call and will
* carry on return the figure indices.  The board buffer must be able to
* hold figureSize*DISTINCT_FIGURE_COUNT(figureSet) ints and will carry
* on return the board's tiling layout as indices into the tiling buffer.
* The return value indicates if a solution has been found (1) or not (0).
*/
int getNextTiling(
	figureSet_t const* figureSet,
	int xDimension,
	int* tiling,
	int* board,
	int continuous,
	int skipSymmetrical);

/** Get tiling count for a given X dimension.
*/
int getTilingCount(
	figureSet_t const* figureSet,
	int xDimension,
	int continuous,
	int skipSymmetrical);


#endif

